//=========================================================
// LPC1114 Project
//=========================================================
// File Name : spi.c
// Function  : SPI Control
//---------------------------------------------------------
// Rev.01 2010.08.07 Munetomo Maruyama
// Rev.A 2014 06.21 Ichiro Shirasaka
//    Add 256Byte Transfer With FIFO mode
//---------------------------------------------------------
// Copyright (C) 2010-2011 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include "gpio.h"
#include "spi.h"

//===================
// Define Constants
//===================
#define SSP_BUFSIZE  16
#define SSP_FIFOSIZE  8
#define FIFO_PRELOAD_BYTE 4
//
/* SSP Status register */
#define SSPSR_TFE       (1 << 0)
#define SSPSR_TNF       (1 << 1)
#define SSPSR_RNE       (1 << 2)
#define SSPSR_RFF       (1 << 3)
#define SSPSR_BSY       (1 << 4)

/* SSP CR0 register */
#define SSPCR0_DSS      (1 << 0)
#define SSPCR0_FRF      (1 << 4)
#define SSPCR0_SPO      (1 << 6)
#define SSPCR0_SPH      (1 << 7)
#define SSPCR0_SCR      (1 << 8)

/* SSP CR1 register */
#define SSPCR1_LBM      (1 << 0)
#define SSPCR1_SSE      (1 << 1)
#define SSPCR1_MS       (1 << 2)
#define SSPCR1_SOD      (1 << 3)

/* SSP Interrupt Mask Set/Clear register */
#define SSPIMSC_RORIM   (1 << 0)
#define SSPIMSC_RTIM    (1 << 1)
#define SSPIMSC_RXIM    (1 << 2)
#define SSPIMSC_TXIM    (1 << 3)

/* SSP0 Interrupt Status register */
#define SSPRIS_RORRIS   (1 << 0)
#define SSPRIS_RTRIS    (1 << 1)
#define SSPRIS_RXRIS    (1 << 2)
#define SSPRIS_TXRIS    (1 << 3)

/* SSP0 Masked Interrupt register */
#define SSPMIS_RORMIS   (1 << 0)
#define SSPMIS_RTMIS    (1 << 1)
#define SSPMIS_RXMIS    (1 << 2)
#define SSPMIS_TXMIS    (1 << 3)

/* SSP0 Interrupt clear register */
#define SSPICR_RORIC    (1 << 0)
#define SSPICR_RTIC     (1 << 1)

// SLOT ASSIGN
extern uint8_t cam_slot;	//MAPLE-mini Slot Value=0:Slot1 Value=1:slot2
extern uint8_t oled_slot;	//MAPLE-mini Slot Value=0:Slot1 Value=1:slot2

//======================
// Initialize SPI
//======================
void Init_SPI(uint32_t bitlen, uint32_t speed, uint32_t spitype)
{
    uint32_t i;
    uint8_t Dummy;

    // Initialize SPI Port
    //     PIO0_2 : SSEL
    //     PIO0_10 : SCLK
    //     PIO0_9 : MOSI
    //     PIO0_8 : MISO
    if (spitype == SPI_GENERIC)
    {
    	GPIOSetDir(0, 16, 1); // PIO0_16 out
    	GPIOSetDir(0, 2, 1); // PIO0_2 out
        LPC_IOCON->PIO0_2  = 0x00000000; // SSEL, disable pu/pd mos
        GPIOSetBitValue(0, 16, 1); // high
        GPIOSetBitValue(0, 2, 1); // high
    }
    else // if (spitype == SPI_SDCARD)
    {
        GPIOSetDir(0, 2, 1); // PIO0_2 out
        LPC_IOCON->PIO0_2  = 0x00000000; // GPIO, disable pu/pd mos
        GPIOSetBitValue(0, 2, 1); // high
    }
    //LPC_IOCON->PIO = 0x00000002; // SCK is PIO0_10
    LPC_IOCON->SWCLK_PIO0_10  = 0x00000002; // SCK, disable pu/pd mos
    LPC_IOCON->PIO0_9 = 0x00000001; // MOSI, disable pu/pd mos
    LPC_IOCON->PIO0_8 = 0x00000011; // MISO, enable pu/pd mos
    //
    // Initialize SPI
    //
    LPC_SYSCON->PRESETCTRL |= (0x1<<0);
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<11);
    LPC_SYSCON->SSP0CLKDIV = 0x01; // Divided by 1

    switch(speed){
    	case SPI_FAST:
    		// Data Length = bitlen
    		// Frame format = SPI, CPOL = 0, CPHA = 0
    		// Bit Frequency = 12MHz, PCLK=48MHz
    		//     CPSDVSR*(SCR+1)=2, SCR=0, CPSR=2
    		LPC_SSP0->CR0 = 0x0000 + (bitlen - 1);
    		LPC_SSP0->CPSR = 2;
    		break;
    	case SPI_MIDDLE:
    	    // Re-Adjust SPI Clock to 12MHz for OLED Module
    	    // Bit Frequency = 12MHz, PCLK=48MHz
    	    //     CPSDVSR*(SCR+1)=4, SCR=0, CPSR=4
       		LPC_SSP0->CR0 = 0x0000 + (bitlen - 1);
    	    LPC_SSP0->CPSR = 4;
    		break;
    }
    // Clear the RxFIFO
    for ( i = 0; i < SSP_FIFOSIZE; i++ ) Dummy = LPC_SSP0->DR;

    // Enable the SSP Interrupt
    NVIC_EnableIRQ(SSP0_IRQn);
    NVIC_SetPriority(SSP0_IRQn, 3);

    // Device select as master, SSP Enabled
    LPC_SSP0->CR1 = SSPCR1_SSE;

    // Enable RXMIS interrupt
  //LPC_SSP0->IMSC = SSPIMSC_RXIM;
}
//
// SSP SWITCHER
//

void Switch_spi(uint32_t bitlen, uint32_t speed, uint32_t spitype)
{

    switch(speed){
    	case SPI_FAST:
    		// Data Length = bitlen
    		// Frame format = SPI, CPOL = 0, CPHA = 0
    		// Bit Frequency = 12MHz, PCLK=48MHz
    		//     CPSDVSR*(SCR+1)=2, SCR=0, CPSR=2
    		LPC_SSP0->CR0 = 0x0000 + (bitlen - 1);
    		LPC_SSP0->CPSR = 2;
    		break;
    	case SPI_MIDDLE:
    	    // Re-Adjust SPI Clock to 12MHz for OLED Module
    	    // Bit Frequency = 12MHz, PCLK=48MHz
    	    //     CPSDVSR*(SCR+1)=4, SCR=0, CPSR=4
       		LPC_SSP0->CR0 = 0x0000 + (bitlen - 1);
    	    LPC_SSP0->CPSR = 4;
    		break;
    }
    return;
}

//========================
// SSP Interrupt Handler
//========================
void SSP0_IRQHandler(void)
{
    uint32_t regValue;

    regValue = LPC_SSP0->MIS;

    // Overrun?
    if (regValue & SSPMIS_RORMIS)
    {
        LPC_SSP0->ICR = SSPICR_RORIC; // Clear
    }
    // Timeout?
    if ( regValue & SSPMIS_RTMIS )
    {
        LPC_SSP0->ICR = SSPICR_RTIC; // Clear
    }
    // Tx FIFO Half Empty?
    if ( regValue & SSPMIS_TXMIS )
    {
        // none
    }
    // Rx FIFO not FULL?
    if ( regValue & SSPMIS_RXMIS )
    {
        // none
        //uint32_t dummy;
        //while (SPI_RxData(&dummy) == 0); //dummy read
    }
}

//===============================
// SPI Transfer and Receive Data
//===============================
uint32_t SPI_TxRxData(uint32_t txdata)
{
    uint32_t rxdata;

    // Tx
   while (!(LPC_SSP0->SR & SSPSR_TNF));
    // Send a byte
    LPC_SSP0->DR = txdata;
    // Wait until the Busy bit is cleared
    //while ( LPC_SSP0->SR & SSPSR_BSY );

    // Rx
    while (!(LPC_SSP0->SR & SSPSR_RNE));
    rxdata = LPC_SSP0->DR;

    // Wait until the Busy bit is cleared
    //while ( LPC_SSP0->SR & SSPSR_BSY );

    return rxdata;
}

//======================
// SPI Transfer Data
//======================
//Long Transfer(over FIFO_PRELOAD_BYTE)
void SPI_LtxData(uint8_t *txdata, uint16_t len, uint8_t slot)
{
	uint16_t n;
	uint8_t data;

	if (slot == 1){				//SLOT2
		LPC_GPIO->W0[2] = 0;	//GPIOSetBitValue(0, 2, 0);  //SEL
		for(n = FIFO_PRELOAD_BYTE;n > 0;n--){
			LPC_SSP0->DR = *txdata++ | 0x100;
		}
		do{
		    while (!(LPC_SSP0->SR & SSPSR_RNE));
		    data = LPC_SSP0->DR;
			if (n++ < len-FIFO_PRELOAD_BYTE) LPC_SSP0->DR = *txdata++ | 0x100;
		}while(n < len);
		LPC_GPIO->W0[2] = 1;	//GPIOSetBitValue(0, 2, 1);  //SEL
	}
	else{
		LPC_GPIO->W0[16] = 0;	//GPIOSetBitValue(0, 16, 0);  //SEL
		for(n = FIFO_PRELOAD_BYTE;n > 0;n--){
			LPC_SSP0->DR = *txdata++ | 0x100;
		}
		do{
		    while (!(LPC_SSP0->SR & SSPSR_RNE));
		    data = LPC_SSP0->DR;
			if (n++ < len-FIFO_PRELOAD_BYTE)LPC_SSP0->DR = *txdata++ | 0x100;
		}while(n < len);
		LPC_GPIO->W0[16] = 1;	//GPIOSetBitValue(0, 16, 1);  //SEL
	}
}

// 1byte Transfer
void SPI_TxData(uint32_t txdata, uint8_t slot)
{
	if (slot == 1){			//SLOT2
		LPC_GPIO->W0[2] = 0;	//GPIOSetBitValue(0, 2, 0);  //SEL
		SPI_TxRxData(txdata);
		LPC_GPIO->W0[2] = 1;	//GPIOSetBitValue(0, 2, 1);  //SEL
	}
	else{
		LPC_GPIO->W0[16] = 0;	//GPIOSetBitValue(0, 16, 0);  //SEL
		SPI_TxRxData(txdata);
		LPC_GPIO->W0[16] = 1;	//GPIOSetBitValue(0, 16, 1);  //SEL
	}
}
//======================
// SPI Receive Data
//======================
// Long Transfer(Over FIFO_PRELOAD_BYTE)
void SPI_LrxData(uint32_t txdata, uint8_t *rxdata, uint16_t len, uint8_t slot)
{
	uint16_t n;
	uint8_t data;

 	if (slot == 1){				//SLOT2
		LPC_GPIO->W0[2] = 0;	//GPIOSetBitValue(0, 2, 0);  //SEL
		for(n = FIFO_PRELOAD_BYTE;n > 0;n--){
			LPC_SSP0->DR = txdata;
		}
		do{
		    while (!(LPC_SSP0->SR & SSPSR_RNE));
		    data = LPC_SSP0->DR;
			if (n++ < len-FIFO_PRELOAD_BYTE) LPC_SSP0->DR = txdata;
		    *rxdata++ = data;
		}while(n < len);
		LPC_GPIO->W0[2] = 1;	//GPIOSetBitValue(0, 2, 1);  //SEL
	}
	else{
		LPC_GPIO->W0[16] = 0;	//GPIOSetBitValue(0, 16, 0);  //SEL
		for(n = FIFO_PRELOAD_BYTE;n > 0;n--){
			LPC_SSP0->DR = txdata;
		}
		do{
		    while (!(LPC_SSP0->SR & SSPSR_RNE));
		    data = LPC_SSP0->DR;
			if (n++ < len-FIFO_PRELOAD_BYTE) LPC_SSP0->DR = txdata;
		    *rxdata++ = data;
		}while(n < len);
		LPC_GPIO->W0[16] = 1;	//GPIOSetBitValue(0, 16, 1);  //SEL
	}
     return;
}

uint32_t SPI_RxData(uint32_t txdata, uint8_t slot)
{
	uint32_t rxdata;
 	if (slot == 1){				//SLOT2
		LPC_GPIO->W0[2] = 0;	//GPIOSetBitValue(0, 2, 0);  //SEL
		rxdata = SPI_TxRxData(txdata);
		LPC_GPIO->W0[2] = 1;	//GPIOSetBitValue(0, 2, 1);  //SEL
	}
	else{
		LPC_GPIO->W0[16] = 0;	//GPIOSetBitValue(0, 16, 0);  //SEL
		rxdata = SPI_TxRxData(txdata);
		LPC_GPIO->W0[16] = 1;	//GPIOSetBitValue(0, 16, 1);  //SEL
	}
    return rxdata;
}

//=========================================================
// End of Program
//=========================================================
